unit JsonForm;

interface

uses
  JsonStdTypes, JsonUserDefinedTypes, SynCommons, mORMot,
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls, Contnrs, ExtCtrls;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    bRecordToJson: TButton;
    bRecordFromJson: TButton;
    bRecordArrayFromJson: TButton;
    bRecordArrayToJson: TButton;
    bObjectToJson: TButton;
    bObjectFromJson: TButton;
    bObjectListToJson: TButton;
    bObjectListFromJson: TButton;
    Panel2: TPanel;
    Memo1: TMemo;
    Memo2: TMemo;
    rgRecordArrayFormat: TRadioGroup;
    rgSimpleArrayFormat: TRadioGroup;
    procedure FormCreate(Sender: TObject);
    procedure rgRecordArrayFormatClick(Sender: TObject);
    procedure bRecordToJsonClick(Sender: TObject);
    procedure bRecordFromJsonClick(Sender: TObject);
    procedure bRecordArrayToJsonClick(Sender: TObject);
    procedure bRecordArrayFromJsonClick(Sender: TObject);
    procedure bObjectToJsonClick(Sender: TObject);
    procedure bObjectFromJsonClick(Sender: TObject);
    procedure bObjectListToJsonClick(Sender: TObject);
    procedure bObjectListFromJsonClick(Sender: TObject);
    procedure rgSimpleArrayFormatClick(Sender: TObject);
  private
  public
    JsonString: RawUTF8;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

//----------------------------------------------------------------------------------------------------------------------

procedure TForm1.FormCreate(Sender: TObject);
begin;
  Randomize;
  rgSimpleArrayFormatClick(nil);
  rgRecordArrayFormatClick(nil);
  end;

procedure TForm1.rgSimpleArrayFormatClick(Sender: TObject);
begin;
  SetJsonSimpleArrayFormat(rgSimpleArrayFormat.ItemIndex, nil);
  end;

procedure TForm1.rgRecordArrayFormatClick(Sender: TObject);
begin;
  SetJsonRecordArrayFormat(rgRecordArrayFormat.ItemIndex, nil);
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure FillRecord(var r: TUserRecord; i: integer);
begin;
  with r do begin;
    rint := i;
    ri64 :=-i;
    rdub := i/10;
    rdt  := i+Now;
    rstr := 'day/'+IntToStr(DayOfWeek(rdt));
    end;
  end;

procedure ShowRecord(const r: TUserRecord; Memo: TMemo);
begin;
  with r do
    Memo.Lines.Add(Format('Record: %d, %d, %f, %s, %s', 
      [rint, ri64, rdub, DateTimeToStr(rdt), rstr]));
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure TForm1.bRecordToJsonClick(Sender: TObject);
var
  r: TUserRecord;
begin;
  FillRecord(r, 365);
  JsonString:=RecordSaveJson(r, TypeInfo(TUserRecord));

  Memo1.Text:=UTF8ToString(JsonString);
  end;

procedure TForm1.bRecordFromJsonClick(Sender: TObject);
var
  r: TUserRecord;
  isValid: boolean;
  s: RawUTF8;
begin;
  Memo2.Lines.Clear;

  if JsonString<>'' then begin;
    SetString(s,pchar(pointer(JsonString)),Length(JsonString));
    isValid:=RecordLoadJson(r, pointer(s), TypeInfo(TUserRecord))<>nil;
    if not isValid then Memo2.Lines.Text:='invalid JSON string'
                   else ShowRecord(r, Memo2);
    end;
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure TForm1.bRecordArrayToJsonClick(Sender: TObject);
var
  ra: TDynRecordArray;
  RecArrCount: integer;
  da: TDynArray;
  i: integer;
  W : TTextWriter;
begin;
  da.Init(TypeInfo(TDynRecordArray), ra, @RecArrCount);
  da.Count:=5;
  for i:=0 to da.Count-1 do FillRecord(ra[i], i);

  W:=TTextWriter.CreateOwnedStream;
  W.AddDynArrayJson(da);
  JsonString:=W.Text;
  W.Free;

  Memo1.Text:=UTF8ToString(JsonString);
  end;

procedure TForm1.bRecordArrayFromJsonClick(Sender: TObject);
var
  ra: TDynRecordArray;
  RecArrCount: integer;
  da: TDynArray;
  i: integer;
  isValid: boolean;
  s: RawUTF8;
begin;
  Memo2.Lines.Clear;

  if JsonString<>'' then begin;
    da.Init(TypeInfo(TDynRecordArray), ra, @RecArrCount);
    SetString(s,pchar(pointer(JsonString)),Length(JsonString));
    isValid:=da.LoadFromJson(pointer(s))<>nil;
    da.Capacity:=da.Count;
    if not isValid then Memo2.Lines.Text:='invalid JSON string'
                   else for i:=0 to Length(ra)-1 do ShowRecord(ra[i], Memo2);
    end;
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure FillSQLObj(SQLObj: TUserClass);
var
  i: integer;
begin;
  //some managed types
  SQLObj.int:=-1;
  SQLObj.i64:=int64(-1) shr 1;
  SQLObj.dub:=Random;
  SQLObj.dt:=Now;
  SQLObj.str:='today';

  //dynamic arrays of some managed types
  SetLength(SQLObj.fIntArr, 2); SQLObj.IntArr[0]:=1; SQLObj.IntArr[1]:=2;
  SetLength(SQLObj.fI64Arr, 2); SQLObj.I64Arr[0]:=3; SQLObj.I64Arr[1]:=4;
  SetLength(SQLObj.fDubArr, 2); SQLObj.DubArr[0]:=5.5; SQLObj.DubArr[1]:=Pi;
  SetLength(SQLObj.fDTArr, 2);  SQLObj.DTArr[0]:=Now-1; SQLObj.DTArr[1]:=Now+1;
  SetLength(SQLObj.fStrArr, 2); SQLObj.StrArr[0]:='yesterday'; SQLObj.StrArr[1]:='tomorrow';

  //dynamic array of records of some managed types
  SetLength(SQLObj.fRecArr, 5);
  for i:=0 to Length(SQLObj.fRecArr)-1 do FillRecord(SQLObj.RecArr[i], i);
  end;

procedure ShowSQLObj(SQLObj: TUserClass; Memo: TMemo);
var
  i: integer;
begin;
  with SQLObj do begin;
    Memo.Lines.Add(Format('Numbers and strings: %d, %d, %f, %s, %s', [int, i64, dub, DateTimeToStr(dt), str]));
    Memo.Lines.Add(Format('Array length: %d, %d, %d, %d, %d, %d',
      [Length(IntArr), Length(I64Arr), Length(DubArr),
       Length(DTArr), Length(StrArr), Length(RecArr)]));
    Memo.Lines.Add(Format('IntArr: %d, %d', [IntArr[0], IntArr[1]]));
    Memo.Lines.Add(Format('I64Arr: %d, %d', [I64Arr[0], I64Arr[1]]));
    Memo.Lines.Add(Format('DubArr: %f, %f', [DubArr[0], DubArr[1]]));
    Memo.Lines.Add(Format('DTArr: %s, %s', [DateTimeToStr(DTArr[0]), DateTimeToStr(DTArr[1])]));
    Memo.Lines.Add(Format('StrArr: %s, %s', [StrArr[0], StrArr[1]]));
    for i:=0 to Length(RecArr)-1 do ShowRecord(RecArr[i], Memo);
    end;
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure TForm1.bObjectToJsonClick(Sender: TObject);
var
  SQLObj: TUserClass;
begin;
  Memo1.Lines.Clear;

  SQLObj:=TUserClass.Create;
  try
    FillSQLObj(SQLObj);
    JsonString:=ObjectToJson(SQLObj, [woStoreClassName]);
    Memo1.Text:=UTF8ToString(JsonString);
  finally
    SQLObj.Free;
    end;
  end;

procedure TForm1.bObjectFromJsonClick(Sender: TObject);
var
  SQLObj: TUserClass;
  isValid: boolean;
  s: RawUTF8;
begin;
  Memo2.Lines.Clear;

  if JsonString<>'' then begin;
    SQLObj:=TUserClass.Create;
    try
      SetString(s,pchar(pointer(JsonString)),Length(JsonString));
      JsonToObject(SQLObj, pointer(s), isValid);
      if not isValid then Memo2.Lines.Text:='invalid JSON string'
                     else ShowSQLObj(SQLObj, Memo2);
    finally
      SQLObj.Free;
      end;
    end;
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure TForm1.bObjectListToJsonClick(Sender: TObject);
var
  List: TObjectList;
begin;
  Memo1.Lines.Clear;

  List:=TObjectList.Create(true); //ObjectList owns objects
  try
    List.Add(TUserClass.Create); FillSQLObj(TUserClass(List[List.Count-1]));
    List.Add(TUserClass.Create); FillSQLObj(TUserClass(List[List.Count-1]));
    JsonString:=ObjectToJson(List, [woStoreClassName]);
    Memo1.Text:=UTF8ToString(JsonString);
  finally
    List.Free;
    end;
  end;

procedure TForm1.bObjectListFromJsonClick(Sender: TObject);
var
  List: TObjectList;
  isValid: boolean;
  i: integer;
  s: RawUTF8;
begin;
  Memo2.Lines.Clear;

  if JsonString<>'' then begin;
    List:=TObjectList.Create(true); //ObjectList owns objects
    try
      SetString(s,pchar(pointer(JsonString)),Length(JsonString));
      JsonToObject(List, pointer(s), isValid);
      if not isValid then Memo2.Lines.Text:='invalid JSON string'
                     else for i:=0 to List.Count-1 do ShowSQLObj(TUserClass(List[i]), Memo2);
    finally
      List.Free;
      end;
    end;
  end;

//----------------------------------------------------------------------------------------------------------------------

end.

