unit JsonStdTypes;

interface


uses
  SynCommons, mORMot, Classes;

type
  TDynInt64Array= array of int64;
  TDynIntegerArray= array of integer;
  TDynSmallIntArray= array of SmallInt;
  TDynShortIntArray= array of ShortInt;
  TDynCardinalArray= array of cardinal;
  TDynWordArray= array of word;
  TDynByteArray= array of byte;
  TDynBooleanArray= array of boolean;
  TDynDoubleArray= array of double;
  TDynDateTimeArray = array of TDateTime;
  TDynStringArray= array of string;

type
  TDynArrayWriter= procedure(aUserInfo: pointer; const aWriter: TTextWriter; const aValue);
  TDynArrayReader= function(aUserInfo: pointer; p: PUTF8Char; var aValue; out aValid: Boolean): PUTF8Char;

procedure RegisterDynArrayRW(aTypeInfo: pointer; aReader: TDynArrayReader; aWriter: TDynArrayWriter; aUserInfo: pointer= nil);
procedure SetJsonSimpleArrayFormat(aFormat: integer; aUserInfo: pointer= nil);

implementation

//----------------------------------------------------------------------------------------------------------------------

procedure RegisterDynArrayRW(aTypeInfo: pointer; aReader: TDynArrayReader; aWriter: TDynArrayWriter; aUserInfo: pointer= nil);
var
  Reader, Writer: TMethod;
begin;
  Reader.Code:=@aReader; Reader.Data:=aUserInfo; if @aReader=nil then Reader.Data:=nil;
  Writer.Code:=@aWriter; Writer.Data:=aUserInfo; if @aWriter=nil then Writer.Data:=nil;
  TTextWriter.RegisterCustomJSONSerializer(aTypeInfo, 
    TDynArrayJSONCustomReader(Reader), 
    TDynArrayJSONCustomWriter(Writer));
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure Int64ArrayWriter(aUserInfo: pointer; const aWriter: TTextWriter; const aValue);
var
  v: int64 absolute aValue;
begin;
  aWriter.Add(v);
  end;

function Int64ArrayReader(aUserInfo: pointer; p: PUTF8Char; var aValue; out aValid: Boolean): PUTF8Char;
var
  v: int64 absolute aValue;
  wasString: boolean;
  delim: AnsiChar;
begin;
  aValid:=false;
  Result:=nil;
  if p=nil then exit;
  v:=GetInt64(GetJSONField(p, p, @wasString, @delim));
  if (p=nil) or wasString then exit;
  dec(p); p^:=delim;
  aValid:=true;
  Result:=p; // ', ' or ']' for last item of array
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure IntegerArrayWriter(aUserInfo: pointer; const aWriter: TTextWriter; const aValue);
var
  v: integer absolute aValue;
begin;
  aWriter.Add(v);
  end;

function IntegerArrayReader(aUserInfo: pointer; p: PUTF8Char; var aValue; out aValid: Boolean): PUTF8Char;
var
  v: integer absolute aValue;
  wasString: boolean;
  delim: AnsiChar;
begin;
  aValid:=false;
  Result:=nil;
  if p=nil then exit;
  v:=GetInteger(GetJSONField(p, p, @wasString, @delim));
  if (p=nil) or wasString then exit;
  dec(p); p^:=delim;
  aValid:=true;
  Result:=p; // ', ' or ']' for last item of array
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure SmallIntArrayWriter(aUserInfo: pointer; const aWriter: TTextWriter; const aValue);
var
  v: SmallInt absolute aValue;
begin;
  aWriter.Add(integer(v));
  end;

function SmallIntArrayReader(aUserInfo: pointer; p: PUTF8Char; var aValue; out aValid: Boolean): PUTF8Char;
var
  v: SmallInt absolute aValue;
  wasString: boolean;
  delim: AnsiChar;
begin;
  aValid:=false;
  Result:=nil;
  if p=nil then exit;
  v:=GetInteger(GetJSONField(p, p, @wasString, @delim));
  if (p=nil) or wasString then exit;
  dec(p); p^:=delim;
  aValid:=true;
  Result:=p; // ', ' or ']' for last item of array
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure ShortIntArrayWriter(aUserInfo: pointer; const aWriter: TTextWriter; const aValue);
var
  v: ShortInt absolute aValue;
begin;
  aWriter.Add(integer(v));
  end;

function ShortIntArrayReader(aUserInfo: pointer; p: PUTF8Char; var aValue; out aValid: Boolean): PUTF8Char;
var
  v: ShortInt absolute aValue;
  wasString: boolean;
  delim: AnsiChar;
begin;
  aValid:=false;
  Result:=nil;
  if p=nil then exit;
  v:=GetInteger(GetJSONField(p, p, @wasString, @delim));
  if (p=nil) or wasString then exit;
  dec(p); p^:=delim;
  aValid:=true;
  Result:=p; // ', ' or ']' for last item of array
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure CardinalArrayWriter(aUserInfo: pointer; const aWriter: TTextWriter; const aValue);
var
  v: cardinal absolute aValue;
begin;
  aWriter.AddU(v);
  end;

function CardinalArrayReader(aUserInfo: pointer; p: PUTF8Char; var aValue; out aValid: Boolean): PUTF8Char;
var
  v: cardinal absolute aValue;
  wasString: boolean;
  delim: AnsiChar;
begin;
  aValid:=false;
  Result:=nil;
  if p=nil then exit;
  v:=GetCardinal(GetJSONField(p, p, @wasString, @delim));
  if (p=nil) or wasString then exit;
  dec(p); p^:=delim;
  aValid:=true;
  Result:=p; // ', ' or ']' for last item of array
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure WordArrayWriter(aUserInfo: pointer; const aWriter: TTextWriter; const aValue);
var
  v: word absolute aValue;
begin;
  aWriter.AddU(cardinal(v));
  end;

function WordArrayReader(aUserInfo: pointer; p: PUTF8Char; var aValue; out aValid: Boolean): PUTF8Char;
var
  v: word absolute aValue;
  wasString: boolean;
  delim: AnsiChar;
begin;
  aValid:=false;
  Result:=nil;
  if p=nil then exit;
  v:=GetCardinal(GetJSONField(p, p, @wasString, @delim));
  if (p=nil) or wasString then exit;
  dec(p); p^:=delim;
  aValid:=true;
  Result:=p; // ', ' or ']' for last item of array
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure ByteArrayWriter(aUserInfo: pointer; const aWriter: TTextWriter; const aValue);
var
  v: byte absolute aValue;
begin;
  aWriter.AddU(cardinal(v));
  end;

function ByteArrayReader(aUserInfo: pointer; p: PUTF8Char; var aValue; out aValid: Boolean): PUTF8Char;
var
  v: byte absolute aValue;
  wasString: boolean;
  delim: AnsiChar;
begin;
  aValid:=false;
  Result:=nil;
  if p=nil then exit;
  v:=GetCardinal(GetJSONField(p, p, @wasString, @delim));
  if (p=nil) or wasString then exit;
  dec(p); p^:=delim;
  aValid:=true;
  Result:=p; // ', ' or ']' for last item of array
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure BooleanArrayWriter(aUserInfo: pointer; const aWriter: TTextWriter; const aValue);
var
  v: boolean absolute aValue;
begin;
  aWriter.AddU(cardinal(v));
  end;

function BooleanArrayReader(aUserInfo: pointer; p: PUTF8Char; var aValue; out aValid: Boolean): PUTF8Char;
var
  v: boolean absolute aValue;
  wasString: boolean;
  delim: AnsiChar;
begin;
  aValid:=false;
  Result:=nil;
  if p=nil then exit;
  byte(v):=GetCardinal(GetJSONField(p, p, @wasString, @delim));
  if (p=nil) or wasString then exit;
  dec(p); p^:=delim;
  aValid:=true;
  Result:=p; // ', ' or ']' for last item of array
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure DoubleArrayWriter(aUserInfo: pointer; const aWriter: TTextWriter; const aValue);
var
  v: double absolute aValue;
begin;
  aWriter.Add('%', [v], twJSONEscape);
  end;

function DoubleArrayReader(aUserInfo: pointer; p: PUTF8Char; var aValue; out aValid: Boolean): PUTF8Char;
var
  v: double absolute aValue;
  wasString: boolean;
  delim: AnsiChar;
  err: integer;
begin;
  aValid:=false;
  Result:=nil;
  if p=nil then exit;
  v:=GetExtended(GetJSONField(p, p, @wasString, @delim), err);
  if (p=nil) or wasString or (err<>0) then exit;
  dec(p); p^:=delim;
  aValid:=true;
  Result:=p; // ', ' or ']' for last item of array
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure DateTimeArrayWriter(aUserInfo: pointer; const aWriter: TTextWriter; const aValue);
var
  v: TDateTime absolute aValue;
begin;
  aWriter.Add('"%"', [DateTimeToIso8601Text(v)], twNone);
  end;

function DateTimeArrayReader(aUserInfo: pointer; p: PUTF8Char; var aValue; out aValid: Boolean): PUTF8Char;
var
  v: TDateTime absolute aValue;
  wasString: boolean;
  delim: AnsiChar;
begin;
  aValid:=false;
  Result:=nil;
  if p=nil then exit;
  v:=Iso8601ToDateTimePUTF8Char(GetJSONField(p, p, @wasString, @delim));
  if (p=nil) or not wasString then exit;
  dec(p); p^:=delim;
  aValid:=true;
  Result:=p; // ', ' or ']' for last item of array
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure StringArrayWriter(aUserInfo: pointer; const aWriter: TTextWriter; const aValue);
var
  v: string absolute aValue;
begin;
  aWriter.Add('"%"', [StringToUTF8(v)], twJSONEscape);
  end;

function StringArrayReader(aUserInfo: pointer; p: PUTF8Char; var aValue; out aValid: Boolean): PUTF8Char;
var
  v: string absolute aValue;
  wasString: boolean;
  delim: AnsiChar;
begin;
  aValid:=false;
  Result:=nil;
  if p=nil then exit;
  v:=UTF8ToString(GetJSONField(p, p, @wasString, @delim));
  if (p=nil) or not wasString then exit;
  dec(p); p^:=delim;
  aValid:=true;
  Result:=p; // ', ' or ']' for last item of array
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure SetJsonSimpleArrayFormat(aFormat: integer; aUserInfo: pointer= nil);
begin;
  case aFormat of
    1: begin; //array of values = ["value1","value2"],...]
       RegisterDynArrayRW(TypeInfo(TDynInt64Array), Int64ArrayReader, Int64ArrayWriter);
       RegisterDynArrayRW(TypeInfo(TDynIntegerArray), IntegerArrayReader, IntegerArrayWriter);
       RegisterDynArrayRW(TypeInfo(TDynSmallIntArray), SmallIntArrayReader, SmallIntArrayWriter);
       RegisterDynArrayRW(TypeInfo(TDynShortIntArray), ShortIntArrayReader, ShortIntArrayWriter);
       RegisterDynArrayRW(TypeInfo(TDynCardinalArray), CardinalArrayReader, CardinalArrayWriter);
       RegisterDynArrayRW(TypeInfo(TDynWordArray), WordArrayReader, WordArrayWriter);
       RegisterDynArrayRW(TypeInfo(TDynByteArray), ByteArrayReader, ByteArrayWriter);
       RegisterDynArrayRW(TypeInfo(TDynBooleanArray), BooleanArrayReader, BooleanArrayWriter);
       RegisterDynArrayRW(TypeInfo(TDynDoubleArray), DoubleArrayReader, DoubleArrayWriter);
       RegisterDynArrayRW(TypeInfo(TDynDateTimeArray), DateTimeArrayReader, DateTimeArrayWriter);
       RegisterDynArrayRW(TypeInfo(TDynStringArray), StringArrayReader, StringArrayWriter);
       end;
    else begin; //0: Base64 (default) = ["base64"]
       RegisterDynArrayRW(TypeInfo(TDynInt64Array), nil, nil);
       RegisterDynArrayRW(TypeInfo(TDynIntegerArray), nil, nil);
       RegisterDynArrayRW(TypeInfo(TDynSmallIntArray), nil, nil);
       RegisterDynArrayRW(TypeInfo(TDynShortIntArray), nil, nil);
       RegisterDynArrayRW(TypeInfo(TDynCardinalArray), nil, nil);
       RegisterDynArrayRW(TypeInfo(TDynWordArray), nil, nil);
       RegisterDynArrayRW(TypeInfo(TDynByteArray), nil, nil);
       RegisterDynArrayRW(TypeInfo(TDynBooleanArray), nil, nil);
       RegisterDynArrayRW(TypeInfo(TDynDoubleArray), nil, nil);
       RegisterDynArrayRW(TypeInfo(TDynDateTimeArray), nil, nil);
       RegisterDynArrayRW(TypeInfo(TDynStringArray), nil, nil);
       end;
    end;
  end;

end.


